/**
  ******************************************************************************
  * @file    I2C\I2C_EEPROM\main.c
  * @author  MCD Application Team
  * @version  V2.2.0
  * @date     30-September-2014
  * @brief   This file contains the main function for I2C EEPROM Read Write example.
  ******************************************************************************
  */ 

// Includes ------------------------------------------------------------------
#include "stm8s.h"
#include "stm8s_eval_i2c_ee.h"
#include "stm8s_eval_lcd.h"

  // * @addtogroup I2C_EEPROM

// Private typedef -----------------------------------------------------------
typedef enum {FAILED = 0, PASSED = !FAILED} TestStatus;

// Private define ------------------------------------------------------------
// Uncomment the following line to enable using LCD screen for messages display 
#define ENABLE_LCD_MSG_DISPLAY

#define sEE_WRITE_ADDRESS1        0x50
#define sEE_READ_ADDRESS1         0x50
#define BUFFER_SIZE1             (countof(Tx1_Buffer)-1)
#define BUFFER_SIZE2             (countof(Tx2_Buffer)-1)
#define sEE_WRITE_ADDRESS2       (sEE_WRITE_ADDRESS1 + BUFFER_SIZE1)
#define sEE_READ_ADDRESS2        (sEE_READ_ADDRESS1 + BUFFER_SIZE1)

// Private macro -------------------------------------------------------------
#define countof(a) (sizeof(a) / sizeof(*(a)))

// Private variables ---------------------------------------------------------
uint8_t Tx1_Buffer[] = "/* STM8S I2C Firmware Library EEPROM driver example: \
                        buffer 1 transfer into address sEE_WRITE_ADDRESS1 */ \
                        Example Description \
                        This firmware provides a basic example of how to use the I2C firmware library and\
                        an associate I2C EEPROM driver to communicate with an I2C EEPROM device (here the\
                        example is interfacing with M24C64 EEPROM)\
                          \
                        I2C peripheral is configured in Master transmitter during write operation and in\
                        Master receiver during read operation from I2C EEPROM. \
                          \
                        I2C peripheral speed is set to 200kHz and can be configured by \
                        modifying the relative define in stm8s_eval_i2c_ee.h file.\
                         \
                        For M24C64 devices all the memory is accessible through the two-bytes \
                        addressing mode and need to define block addresses. In this case, only the physical \
                        address has to be defined (according to the address pins (E0,E1 and E2) connection).\
                        This address is defined in stm8s_eval_i2c_ee.h (default is 0xA0: E0, E1 and E2 tied to ground).\
                        The EEPROM addresses where the program start the write and the read operations \
                        is defined in the main.c file. \
                         \
                        First, the content of Tx1_Buffer is written to the EEPROM_WriteAddress1 and the\
                        written data are read. The written and the read buffers data are then compared.\
                        Following the read operation, the program waits that the EEPROM reverts to its \
                        Standby state. A second write operation is, then, performed and this time, Tx2_Buffer\
                        is written to EEPROM_WriteAddress2, which represents the address just after the last \
                        written one in the first write. After completion of the second write operation, the \
                        written data are read. The contents of the written and the read buffers are compared.\
                         ";
uint8_t Tx2_Buffer[] = "/* STM8S I2C Firmware Library EEPROM driver example: \
                        buffer 2 transfer into address sEE_WRITE_ADDRESS2 */";
uint8_t Rx1_Buffer[BUFFER_SIZE1], Rx2_Buffer[BUFFER_SIZE2];
volatile TestStatus TransferStatus1 = FAILED, TransferStatus2 = FAILED;
volatile uint16_t NumDataRead = 0;

// Private functions ---------------------------------------------------------
TestStatus Buffercmp(uint8_t* pBuffer1, uint8_t* pBuffer2, uint16_t BufferLength);
/**
  * @brief  Main program
  * @param  None
  * @retval None
  */
void main(void)
{
  // Initialization of the clock 
  // Clock divider to HSI/1 
  CLK_HSIPrescalerConfig(CLK_PRESCALER_HSIDIV1);

#ifdef ENABLE_LCD_MSG_DISPLAY
  // Initialize the LCD screen for information display 
  STM8S_EVAL_LCD_Init();
  LCD_Clear();  

  LCD_SetCursorPos(LCD_LINE1, 0);
  LCD_Print("STM8S FW Library");
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("   I2C EEPROM   ");
#endif /* ENABLE_LCD_MSG_DISPLAY */  
  
  // Initialize the I2C EEPROM driver ----------------------------------------
  sEE_Init();  

  // First write in the memory followed by a read of the written data --------
  // Write on I2C EEPROM from sEE_WRITE_ADDRESS1 
  sEE_WriteBuffer(Tx1_Buffer, sEE_WRITE_ADDRESS1, BUFFER_SIZE1); 

  // Wait for EEPROM standby state 
  sEE_WaitEepromStandbyState();  
  
  // Set the Number of data to be read 
  NumDataRead = BUFFER_SIZE1;
  
  // Read from I2C EEPROM from sEE_READ_ADDRESS1 
  sEE_ReadBuffer(Rx1_Buffer, sEE_READ_ADDRESS1, (uint16_t *)(&NumDataRead)); 

 
#ifdef ENABLE_LCD_MSG_DISPLAY  
  LCD_SetCursorPos(LCD_LINE1,0);
  LCD_Print(" EEPROM Transfer1");
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("     Ongoing     ");
#endif /* ENABLE_LCD_MSG_DISPLAY */ 
  
  // Wait till transfer is complete 
  while (NumDataRead > 0)
  {}  
  
  // Check if the data written to the memory is read correctly 
  TransferStatus1 = Buffercmp(Tx1_Buffer, Rx1_Buffer, BUFFER_SIZE1);
  /* TransferStatus1 = PASSED, if the transmitted and received data 
     to/from the EEPROM are the same */
  /* TransferStatus1 = FAILED, if the transmitted and received data 
     to/from the EEPROM are different */
#ifdef ENABLE_LCD_MSG_DISPLAY  
  if (TransferStatus1 == PASSED)
  {
    LCD_SetCursorPos(LCD_LINE1,0);
    LCD_Print(" EEPROM Transfer1");
    LCD_SetCursorPos(LCD_LINE2, 0);
    LCD_Print("     PASSED      ");
  }
  else
  {
    LCD_SetCursorPos(LCD_LINE1,0);
    LCD_Print(" EEPROM Transfer1");
    LCD_SetCursorPos(LCD_LINE2, 0);
    LCD_Print("     FAILED      ");
  }  
#endif /* ENABLE_LCD_MSG_DISPLAY */  

  // Second write in the memory followed by a read of the written data -------
  // Write on I2C EEPROM from sEE_WRITE_ADDRESS2 
  sEE_WriteBuffer(Tx2_Buffer, sEE_WRITE_ADDRESS2, BUFFER_SIZE2); 

  // Wait for EEPROM standby state 
  sEE_WaitEepromStandbyState();  
  
  // Set the Number of data to be read 
  NumDataRead = BUFFER_SIZE2;  
  
  // Read from I2C EEPROM from sEE_READ_ADDRESS2 
  sEE_ReadBuffer(Rx2_Buffer, sEE_READ_ADDRESS2, (uint16_t *)(&NumDataRead));


#ifdef ENABLE_LCD_MSG_DISPLAY   
  LCD_SetCursorPos(LCD_LINE1,0);
  LCD_Print(" EEPROM Transfer2");
  LCD_SetCursorPos(LCD_LINE2, 0);
  LCD_Print("     Ongoing     ");
#endif /* ENABLE_LCD_MSG_DISPLAY */  
  
  // Wait till transfer is complete 
  while (NumDataRead > 0)
  {}
  
  // Check if the data written to the memory is read correctly 
  TransferStatus2 = Buffercmp(Tx2_Buffer, Rx2_Buffer, BUFFER_SIZE2);
  /* TransferStatus2 = PASSED, if the transmitted and received data 
     to/from the EEPROM are the same */
  /* TransferStatus2 = FAILED, if the transmitted and received data 
     to/from the EEPROM are different */
#ifdef ENABLE_LCD_MSG_DISPLAY   
  if (TransferStatus1 == PASSED)
  {
    LCD_SetCursorPos(LCD_LINE1,0);
    LCD_Print(" EEPROM Transfer2");
    LCD_SetCursorPos(LCD_LINE2, 0);
    LCD_Print("     PASSED      ");
  }
  else
  {
    LCD_SetCursorPos(LCD_LINE1,0);
    LCD_Print(" EEPROM Transfer2");
    LCD_SetCursorPos(LCD_LINE2, 0);
    LCD_Print("     FAILED      ");
  }  
#endif /* ENABLE_LCD_MSG_DISPLAY */
  
  // Free all used resources 
  sEE_DeInit();

  while (1)
  {
  }
}

#ifndef USE_DEFAULT_TIMEOUT_CALLBACK
/**
  * @brief  Example of timeout situation management.
  * @param  None.
  * @retval None.
  */
uint32_t sEE_TIMEOUT_UserCallback(void)
{
  /* User application may try to recover the communication by resetting I2C
    peripheral (calling the function I2C_SoftwareResetCmd()) then re-start
    the transmission/reception from a previously stored recover point.
    For simplicity reasons, this example only shows a basic way for errors 
    managements which consists of stopping all the process and requiring system
    reset. */
  
#ifdef ENABLE_LCD_MSG_DISPLAY   
  // Display error message on screen 
    LCD_SetCursorPos(LCD_LINE1,0);
    LCD_Print("    Com ERROR    ");
    LCD_SetCursorPos(LCD_LINE2, 0);
    LCD_Print("   Reset Board   ");
#endif /* ENABLE_LCD_MSG_DISPLAY */
  
  // Block communication and all processes 
  while (1)
  {   
  }  
}
#endif /* USE_DEFAULT_TIMEOUT_CALLBACK */

/**
  * @brief  Compares two buffers.
  * @param  pBuffer1, pBuffer2: buffers to be compared.
  * @param  BufferLength: buffer's length
  * @retval PASSED: pBuffer1 identical to pBuffer2
  *         FAILED: pBuffer1 differs from pBuffer2
  */
TestStatus Buffercmp(uint8_t* pBuffer1, uint8_t* pBuffer2, uint16_t BufferLength)
{
  while(BufferLength--)
  {
    if(*pBuffer1 != *pBuffer2)
    {
      return FAILED;
    }
    
    pBuffer1++;
    pBuffer2++;
  }

  return PASSED;  
}

#ifdef USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  // Infinite loop 
  while (1)
  {
  }
}
#endif



