/**
  ******************************************************************************
  * @file    IWDG_Example\main.c
  * @author  MCD Application Team
  * @version  V2.2.0
  * @date     30-September-2014
  * @brief   This file contains the main function for IWDG Reset example.
  ******************************************************************************
  */ 

// Includes ------------------------------------------------------------------
#include "stm8s.h"
#include "stm8s_eval.h"

  // * @addtogroup IWDG_Example

// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
// Private macro -------------------------------------------------------------
// Private variables ---------------------------------------------------------
__IO uint32_t LsiFreq = 0;
// Private function prototypes -----------------------------------------------
static void CLK_Config(void);
static void GPIO_Config(void);
static void IWDG_Config(void);
static uint32_t LSIMeasurment(void);
// Private functions ---------------------------------------------------------

/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
void main(void)
{
  // Clock Configuration 
  CLK_Config(); 
      
  // GPIO Configuration 
  GPIO_Config();

  // Check if the system has resumed from IWDG reset 
  if (RST_GetFlagStatus(RST_FLAG_IWDGF) != RESET)
  {
    // IWDGF flag set 
    // Turn on LED1 
    STM_EVAL_LEDOn(LED1);

    // Clear IWDGF Flag 
    RST_ClearFlag(RST_FLAG_IWDGF);
  }
  else
  {
    // IWDGF flag is not set 
    // Turn off LED1 
    STM_EVAL_LEDOff(LED1);
  }

  // Get measured LSI frequency 
  LsiFreq = LSIMeasurment();  
  
  // IWDG Configuration 
  IWDG_Config();
  
  while (1)
  {
    // Reload IWDG counter 
    IWDG_ReloadCounter();  
  }
}

/**
  * @brief  Configure system clock to run at 16Mhz
  * @param  None
  * @retval None
  */
static void CLK_Config(void)
{
  // Initialization of the clock 
  // Clock divider to HSI/1 
  CLK_HSIPrescalerConfig(CLK_PRESCALER_HSIDIV1);
}

/**
  * @brief  Configures the IWDG to generate a Reset if it is not refreshed at the
  *         correct time. 
  * @param  None
  * @retval None
  */
static void IWDG_Config(void)
{
  // Enable IWDG (the LSI oscillator will be enabled by hardware) 
  IWDG_Enable();
  
  /* IWDG timeout equal to 250 ms (the timeout may varies due to LSI frequency
     dispersion) */
  // Enable write access to IWDG_PR and IWDG_RLR registers 
  IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);
  
  // IWDG counter clock: LSI/128 
  IWDG_SetPrescaler(IWDG_Prescaler_128);
  
  /* Set counter reload value to obtain 250ms IWDG Timeout.
    Counter Reload Value = 250ms/IWDG counter clock period
                         = 250ms / (LSI/128)
                         = 0.25s / (LsiFreq/128)
                         = LsiFreq/(128 * 4)
                         = LsiFreq/512
   */
  IWDG_SetReload((uint8_t)(LsiFreq/512));
  
  // Reload IWDG counter 
  IWDG_ReloadCounter();
}

/**
  * @brief  Initialize LED1 and Key Button mounted on the evaluation board
  * @param  None
  * @retval None
  */
static void GPIO_Config(void) 
{
  // Configure LED1 
  STM_EVAL_LEDInit(LED1);
  STM_EVAL_LEDOff(LED1);

  // Configure Key button 
  STM_EVAL_PBInit(BUTTON_KEY, BUTTON_MODE_EXTI);
  
  // enable interrupts 
  enableInterrupts();
}

/**
  * @brief  Measure the LSI frequency using timer IC1 and update the calibration registers.
  * @note   It is recommended to use a timer clock frequency of at least 10MHz in order 
  *         to obtain a better in the LSI frequency measurement.
  * @param  None
  * @retval None
  */
static uint32_t LSIMeasurment(void)
{
  uint32_t lsi_freq_hz = 0x0;
  uint32_t fmaster = 0x0;
  uint16_t ICValue1 = 0x0;
  uint16_t ICValue2 = 0x0;

  // Get master frequency 
  fmaster = CLK_GetClockFreq();

  // Enable the LSI measurement: LSI clock connected to timer Input Capture 1 
  AWU->CSR |= AWU_CSR_MSR;

#if defined (STM8S903) || defined (STM8S103) || defined (STM8S003)
  // Measure the LSI frequency with TIMER Input Capture 1 
  
  // Capture only every 8 events!!! 
  // Enable capture of TI1 
  TIM1_ICInit(TIM1_CHANNEL_1, TIM1_ICPOLARITY_RISING, TIM1_ICSELECTION_DIRECTTI,
              TIM1_ICPSC_DIV8, 0);
  
  // Enable TIM1 
  TIM1_Cmd(ENABLE);
  
  // wait a capture on cc1 
  while((TIM1->SR1 & TIM1_FLAG_CC1) != TIM1_FLAG_CC1);
  // Get CCR1 value
  ICValue1 = TIM1_GetCapture1();
  TIM1_ClearFlag(TIM1_FLAG_CC1);
  
  // wait a capture on cc1 
  while((TIM1->SR1 & TIM1_FLAG_CC1) != TIM1_FLAG_CC1);
  // Get CCR1 value
  ICValue2 = TIM1_GetCapture1();
  TIM1_ClearFlag(TIM1_FLAG_CC1);
  
  // Disable IC1 input capture 
  TIM1->CCER1 &= (uint8_t)(~TIM1_CCER1_CC1E);
  // Disable timer2 
  TIM1_Cmd(DISABLE);
  
#else  
  // Measure the LSI frequency with TIMER Input Capture 1 
  
  // Capture only every 8 events!!! 
  // Enable capture of TI1 
  TIM3_ICInit(TIM3_CHANNEL_1, TIM3_ICPOLARITY_RISING, TIM3_ICSELECTION_DIRECTTI,
              TIM3_ICPSC_DIV8, 0);

  // Enable TIM3 
  TIM3_Cmd(ENABLE);

  // wait a capture on cc1 
  while ((TIM3->SR1 & TIM3_FLAG_CC1) != TIM3_FLAG_CC1);
  // Get CCR1 value
  ICValue1 = TIM3_GetCapture1();
  TIM3_ClearFlag(TIM3_FLAG_CC1);

  // wait a capture on cc1 
  while ((TIM3->SR1 & TIM3_FLAG_CC1) != TIM3_FLAG_CC1);
    // Get CCR1 value
  ICValue2 = TIM3_GetCapture1();
  TIM3_ClearFlag(TIM3_FLAG_CC1);

  // Disable IC1 input capture 
  TIM3->CCER1 &= (uint8_t)(~TIM3_CCER1_CC1E);
  // Disable timer3 
  TIM3_Cmd(DISABLE);
#endif

  // Compute LSI clock frequency 
  lsi_freq_hz = (8 * fmaster) / (ICValue2 - ICValue1);
  
  // Disable the LSI measurement: LSI clock disconnected from timer Input Capture 1 
  AWU->CSR &= (uint8_t)(~AWU_CSR_MSR);

 return (lsi_freq_hz);
}

#ifdef USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  // Infinite loop 
  while (1)
  {
  }
}
#endif



